<?php
/*
Plugin Name: FraudLabs Pro for Easy Digital Downloads
Plugin URI: https://www.fraudlabspro.com
Description: This plugin is an integration for Easy Digital Downloads plugin that help you to screen your order transaction for online fraud.
Author: FraudLabs Pro
Author URI: https://www.fraudlabspro.com/
Version: 1.0.0
*/

defined( 'DS' ) or define( 'DS', DIRECTORY_SEPARATOR );
define( 'FRAUDLABS_PRO_ROOT', __DIR__ . DS );

if( class_exists( 'EDD_License' ) ) {
	$edd_fraudlabspro_license = new EDD_License( __FILE__, 'FraudLabs Pro', '1.0.0', 'FraudLabs Pro', 'fraudlabspro_license_key' );
}

add_action( 'edd_checkout_error_checks', 'edd_fraudlabspro_process_purchase', 10, 2 );
add_action( 'edd_checkout_form_bottom', 'edd_fraudlabspro_add_javascript_agent' );
add_action( 'edd_payment_view_details', 'edd_fraudlabspro_payment_details' );
add_filter( 'edd_settings_extensions', 'edd_fraudlabspro_add_settings' );

// Validate the order during purchase.
function edd_fraudlabspro_process_purchase( $data, $post ) {
	global $edd_options;

	if( empty( $edd_options['fraudlabspro_api_key'] ) ) {
		write_debug_log( 'FraudLabs Pro API Key is empty in Extensions Settings. FraudLabs Pro validation will not be performed.' );
		return;
	}

	$user = edd_get_purchase_form_user( $data );

	if($user['user_email'] == '')
		return;

	write_debug_log( 'FraudLabs Pro validation started.' );

	$url = 'https://api.fraudlabspro.com/v1/order/screen?' . http_build_query( array(
		'key'							=> $edd_options['fraudlabspro_api_key'],
		'format'						=> 'json',
		'ip'							=> ( filter_var( $edd_options['fraudlabspro_test_ip'], FILTER_VALIDATE_IP ) ) ? $edd_options['fraudlabspro_test_ip'] : edd_get_ip(),
		'first_name'					=> $user['user_first'],
		'last_name'						=> $user['user_last'],
		'bill_addr'						=> ( ! empty( $user['address'] ) ) ? $user['address'] : '',
		'bill_city'						=> $user['address']['city'],
		'bill_state'					=> $user['address']['state'],
		'bill_zip_code'					=> $user['address']['zip'],
		'bill_country'					=> $user['address']['country'],
		'email'							=> $user['user_email'],
		'email_domain'					=> substr( $user['user_email'], strpos( $user['user_email'], '@' ) + 1 ),
		'email_hash'					=> hash_string( $user['user_email'] ),
		'amount'						=> edd_get_cart_total(),
		'quantity'						=> edd_get_cart_quantity(),
		'currency'						=> edd_get_currency(),
		'payment_mode'					=> $data['gateway'],
		'flp_checksum'					=> ( isset( $_COOKIE['flp_checksum'] ) ) ? $_COOKIE['flp_checksum'] : '',
		'source'						=> 'easydigitaldownloads',
		'source_version'				=> '1.0.0',
	) );

	$request = wp_remote_get( $url, array( 'timeout' => 10, ) );

	// Give up fraud check if having network issue
	if( is_wp_error( $request ) ) {
		write_debug_log( 'FraudLabs Pro validation skipped. Error during order validation. Error: ' . $request->get_error_message() );
		$reason = __( 'Fraud check skipped. Error: [ ' . $request->get_error_message() . ']', 'edd_fraudlabspro' );
		edd_set_error( '_fraudlabspro' , $reason );
		return;
	}

	// Get the HTTP response
		$response = json_decode( wp_remote_retrieve_body( $request ) );

	// Make sure response is an object
	if ( ! is_object( $response ) ) {
		write_debug_log( 'FraudLabs Pro validation skipped due to network issue.' );
		$reason = __( 'Fraud check skipped due network issue.', 'edd_fraudlabspro' );
		edd_set_error( '_fraudlabspro' , $reason );
		return;
	}

	$GLOBALS['edd_fraudlabspro_data'] = $response;
	add_action( 'edd_insert_payment', function( $payment ) { return edd_fraudlabspro_add_payment_meta( $payment ); } );

	if( $response->fraudlabspro_status == 'APPROVE' ) {
		if( !empty($edd_options['fraudlabspro_approve_status']) ) {
			add_action( 'edd_update_payment_status', 'edd_fraudlabspro_update_payment_status_approve', 10, 3 );
		}
	}
	elseif( $response->fraudlabspro_status == 'REVIEW' ) {
		if( !empty($edd_options['fraudlabspro_review_status']) ) {
			add_action( 'edd_update_payment_status', 'edd_fraudlabspro_update_payment_status_review', 10, 3 );
		}
	}
	elseif( $response->fraudlabspro_status == 'REJECT' ) {
		if( !empty($edd_options['fraudlabspro_reject_status']) ) {
			add_action( 'edd_update_payment_status', 'edd_fraudlabspro_update_payment_status_reject', 10, 3 );
		}
	}

}

// Hash a string to send to FraudLabs Pro API.
function hash_string( $s ) {
	$hash = 'fraudlabspro_' . $s;
	for( $i = 0; $i < 65536; $i++ )
		$hash = sha1( 'fraudlabspro_' . $hash );
	return $hash;
}

// Save FraudLabs Pro Response data to post_meta.
function edd_fraudlabspro_add_payment_meta( $payment ) {
	global $edd_options;
	write_debug_log( 'FraudLabs Pro validation done for Order ' . $payment . '.' );

	update_post_meta( $payment, '_edd_fraudlabspro', json_encode( array(
		'order_id'						=> $payment,
		'is_country_match'				=> $GLOBALS['edd_fraudlabspro_data']->is_country_match,
		'is_high_risk_country'			=> $GLOBALS['edd_fraudlabspro_data']->is_high_risk_country,
		'distance_in_km'				=> $GLOBALS['edd_fraudlabspro_data']->distance_in_km,
		'distance_in_mile'				=> $GLOBALS['edd_fraudlabspro_data']->distance_in_mile,
		'ip_address'					=> ( filter_var( $edd_options['fraudlabspro_test_ip'], FILTER_VALIDATE_IP ) ) ? $edd_options['fraudlabspro_test_ip'] : edd_get_ip(),
		'ip_country'					=> $GLOBALS['edd_fraudlabspro_data']->ip_country,
		'ip_region'						=> $GLOBALS['edd_fraudlabspro_data']->ip_region,
		'ip_city'						=> $GLOBALS['edd_fraudlabspro_data']->ip_city,
		'ip_continent'					=> $GLOBALS['edd_fraudlabspro_data']->ip_continent,
		'ip_latitude'					=> $GLOBALS['edd_fraudlabspro_data']->ip_latitude,
		'ip_longitude'					=> $GLOBALS['edd_fraudlabspro_data']->ip_longitude,
		'ip_timezone'					=> $GLOBALS['edd_fraudlabspro_data']->ip_timezone,
		'ip_elevation'					=> $GLOBALS['edd_fraudlabspro_data']->ip_elevation,
		'ip_domain'						=> $GLOBALS['edd_fraudlabspro_data']->ip_domain,
		'ip_mobile_mnc'					=> $GLOBALS['edd_fraudlabspro_data']->ip_mobile_mnc,
		'ip_mobile_mcc'					=> $GLOBALS['edd_fraudlabspro_data']->ip_mobile_mcc,
		'ip_mobile_brand'				=> $GLOBALS['edd_fraudlabspro_data']->ip_mobile_brand,
		'ip_netspeed'					=> $GLOBALS['edd_fraudlabspro_data']->ip_netspeed,
		'ip_isp_name'					=> $GLOBALS['edd_fraudlabspro_data']->ip_isp_name,
		'ip_usage_type'					=> $GLOBALS['edd_fraudlabspro_data']->ip_usage_type,
		'is_free_email'					=> $GLOBALS['edd_fraudlabspro_data']->is_free_email,
		'is_new_domain_name'			=> $GLOBALS['edd_fraudlabspro_data']->is_new_domain_name,
		'is_proxy_ip_address'			=> $GLOBALS['edd_fraudlabspro_data']->is_proxy_ip_address,
		'is_bin_found'					=> $GLOBALS['edd_fraudlabspro_data']->is_bin_found,
		'is_bin_country_match'			=> $GLOBALS['edd_fraudlabspro_data']->is_bin_country_match,
		'is_bin_name_match'				=> $GLOBALS['edd_fraudlabspro_data']->is_bin_name_match,
		'is_bin_phone_match'			=> $GLOBALS['edd_fraudlabspro_data']->is_bin_phone_match,
		'is_bin_prepaid'				=> $GLOBALS['edd_fraudlabspro_data']->is_bin_prepaid,
		'is_address_ship_forward'		=> $GLOBALS['edd_fraudlabspro_data']->is_address_ship_forward,
		'is_bill_ship_city_match'		=> $GLOBALS['edd_fraudlabspro_data']->is_bill_ship_city_match,
		'is_bill_ship_state_match'		=> $GLOBALS['edd_fraudlabspro_data']->is_bill_ship_state_match,
		'is_bill_ship_country_match'	=> $GLOBALS['edd_fraudlabspro_data']->is_bill_ship_country_match,
		'is_bill_ship_postal_match'		=> $GLOBALS['edd_fraudlabspro_data']->is_bill_ship_postal_match,
		'is_ip_blacklist'				=> $GLOBALS['edd_fraudlabspro_data']->is_ip_blacklist,
		'is_email_blacklist'			=> $GLOBALS['edd_fraudlabspro_data']->is_email_blacklist,
		'is_credit_card_blacklist'		=> $GLOBALS['edd_fraudlabspro_data']->is_credit_card_blacklist,
		'is_device_blacklist'			=> $GLOBALS['edd_fraudlabspro_data']->is_device_blacklist,
		'is_user_blacklist'				=> $GLOBALS['edd_fraudlabspro_data']->is_user_blacklist,
		'fraudlabspro_score'			=> $GLOBALS['edd_fraudlabspro_data']->fraudlabspro_score,
		'fraudlabspro_distribution'		=> $GLOBALS['edd_fraudlabspro_data']->fraudlabspro_distribution,
		'fraudlabspro_status'			=> $GLOBALS['edd_fraudlabspro_data']->fraudlabspro_status,
		'fraudlabspro_id'				=> $GLOBALS['edd_fraudlabspro_data']->fraudlabspro_id,
		'fraudlabspro_error_code'		=> $GLOBALS['edd_fraudlabspro_data']->fraudlabspro_error_code,
		'fraudlabspro_message'			=> $GLOBALS['edd_fraudlabspro_data']->fraudlabspro_message,
		'fraudlabspro_credits'			=> $GLOBALS['edd_fraudlabspro_data']->fraudlabspro_credits,
		'api_key'						=> $edd_options['fraudlabspro_api_key'],
	) ) );
}

// Update payment status when order in FraudLabs Pro Approve status.
function edd_fraudlabspro_update_payment_status_approve( $payment, $new_status, $old_status ) {
	global $edd_options;
	$new_status = $edd_options['fraudlabspro_approve_status'];
	if( $new_status != $old_status ){
		edd_update_payment_status( $payment, $new_status );
	}
}

// Update payment status when order in FraudLabs Pro Review status.
function edd_fraudlabspro_update_payment_status_review( $payment, $new_status, $old_status ) {
	global $edd_options;
	$new_status = $edd_options['fraudlabspro_review_status'];
	if( $new_status != $old_status ){
		edd_update_payment_status( $payment, $new_status );
	}
}

// Update payment status when order in FraudLabs Pro Reject status.
function edd_fraudlabspro_update_payment_status_reject( $payment, $new_status, $old_status ) {
	global $edd_options;
	$new_status = $edd_options['fraudlabspro_reject_status'];
	if( $new_status != $old_status ){
		edd_update_payment_status( $payment, $new_status );
	}
}

// Append FraudLabs Pro Report to order details.
function edd_fraudlabspro_payment_details( $payment ) {
	global $edd_options;

	if( isset( $_POST['approve'] ) ) {
		$request = wp_remote_get( 'https://api.fraudlabspro.com/v1/order/feedback?' . http_build_query( array(
			'key'		=> $edd_options['fraudlabspro_api_key'],
			'action'	=> 'APPROVE',
			'id'		=> $_POST['transactionId'],
			'format'	=> 'json'
		) ) );

		if( ! is_wp_error( $request ) ) {
			// Get the HTTP response
			$response = json_decode( wp_remote_retrieve_body( $request ) );

			if( is_object( $response ) ) {
				if( $response->fraudlabspro_error_code == '' || $response->fraudlabspro_error_code == '304' || $response->fraudlabspro_error_code == '305' ) {
					if( !empty($edd_options['fraudlabspro_approve_status']) ) {
						edd_update_payment_status( $payment, $edd_options['fraudlabspro_approve_status'] );
						echo '<script>window.location.href = window.location.href;</script>';
					}

					$result = get_post_meta( $payment, '_edd_fraudlabspro' );
					$row = json_decode( $result[0] );
					$row->fraudlabspro_status = 'APPROVE';
					update_post_meta( $payment, '_edd_fraudlabspro', json_encode( $row ) );
				}
			}
		}
	}

	if( isset( $_POST['reject'] ) ) {
		$request = wp_remote_get( 'https://api.fraudlabspro.com/v1/order/feedback?' . http_build_query( array(
			'key'		=> $edd_options['fraudlabspro_api_key'],
			'action'	=> 'REJECT',
			'id'		=> $_POST['transactionId'],
			'format'	=> 'json'
		) ) );

		if( ! is_wp_error( $request ) ) {
			// Get the HTTP response
			$response = json_decode( wp_remote_retrieve_body( $request ) );

			if( is_object( $response ) ) {
				if( $response->fraudlabspro_error_code == '' || $response->fraudlabspro_error_code == '304' || $response->fraudlabspro_error_code == '306' ) {
					if( !empty($edd_options['fraudlabspro_reject_status']) ) {
						edd_update_payment_status( $payment, $edd_options['fraudlabspro_reject_status'] );
						echo '<script>window.location.href = window.location.href;</script>';
					}

					$result = get_post_meta( $payment, '_edd_fraudlabspro' );
					$row = json_decode( $result[0] );
					$row->fraudlabspro_status = 'REJECT';
					update_post_meta( $payment, '_edd_fraudlabspro', json_encode( $row ) );
				}
			}
		}
	}

	$reject_blacklist = ( isset ( $_POST['new_status'] ) ) ? $_POST['new_status'] : '';
	if( $reject_blacklist === 'reject_blacklist') {
		$request = wp_remote_get( 'https://api.fraudlabspro.com/v1/order/feedback?' . http_build_query( array(
			'key'		=> $edd_options['fraudlabspro_api_key'],
			'action'	=> 'REJECT_BLACKLIST',
			'id'		=> $_POST['transactionId'],
			'format'	=> 'json',
			'note'		=> $_POST['feedback_note']
		) ) );

		if( ! is_wp_error( $request ) ) {
			// Get the HTTP response
			$response = json_decode( wp_remote_retrieve_body( $request ) );

			if( is_object( $response ) ) {
				if( $response->fraudlabspro_error_code == '' || $response->fraudlabspro_error_code == '304' || $response->fraudlabspro_error_code == '306' ) {
					if( !empty($edd_options['fraudlabspro_reject_status']) ) {
						edd_update_payment_status( $payment, $edd_options['fraudlabspro_reject_status'] );
						echo '<script>window.location.href = window.location.href;</script>';
					}

					$result = get_post_meta( $payment, '_edd_fraudlabspro' );
					$row = json_decode( $result[0] );
					$row->fraudlabspro_status = 'REJECT';
					$row->is_blacklisted = '1';
					update_post_meta( $payment, '_edd_fraudlabspro', json_encode( $row ) );
				}
			}
		}
	}

	$result = get_post_meta( $payment, '_edd_fraudlabspro' );

	if( count( $result ) > 0 ) {
		if( strpos( $result[0], '\\' ) ) {
			$result[0] = str_replace( '\\', '', $result[0] );
		}

		$row = json_decode( $result[0] );
		if( $row->fraudlabspro_id != '' ) {
			$table = '
				<style type="text/css">
					.fraudlabspro {width:100%;}
					.fraudlabspro td{padding:10px 0; vertical-align:top}
					.flp-helper{text-decoration:none}

					/* color: Approve - #45b6af, Reject - #f3565d, Review - #dfba49 */
				</style>

				<table class="fraudlabspro">
					<tr>
						<td colspan="2" style="text-align:center; background-color:#ab1b1c; border:1px solid #ab1b1c; padding-top:10px; padding-bottom:10px;">
							<a href="https://www.fraudlabspro.com" target="_blank"><img src="https://www.fraudlabspro.com/images/logo_200.png" alt="FraudLabs Pro" /></a>
						</td>
					</tr>';

				$location = array();
				if( strlen( $row->ip_country ) == 2 ) {
					$location = array(
						$row->ip_continent,
						get_country_by_code( $row->ip_country ),
						$row->ip_region,
						$row->ip_city
					);

					$location = array_unique( $location );
				}

				switch( $row->fraudlabspro_status ) {
					case 'REVIEW':
						$fraudlabspro_status_display = "REVIEW";
						$color = 'dfba49';
						break;

					case 'REJECT':
						$fraudlabspro_status_display = "REJECTED";
						$color = 'f3565d';
						break;

					case 'APPROVE':
						$fraudlabspro_status_display = "APPROVED";
						$color = '45b6af';
						break;
				}

			$table .= '
				<tr>
					<td style="width:50%;" rowspan="2">
						<b>FraudLabs Pro Score</b> <a href="javascript:;" class="flp-helper" title="Risk score, 0 (low risk) - 100 (high risk)."><span class="dashicons dashicons-editor-help"></span></a><br/>
						<img class="img-responsive" alt="" src="//fraudlabspro.hexa-soft.com/images/fraudscore/' . ( ( $row->fraudlabspro_score ) ? 'fraudlabsproscore' . $row->fraudlabspro_score . '.png' : 'nofraudprotection.png' ) . '" style="width:160px;" />
					</td>
					<td style="width:50%;">
						<b>FraudLabs Pro Status</b> <a href="javascript:;" class="flp-helper" title="FraudLabs Pro status."><span class="dashicons dashicons-editor-help"></span></a>
						<span style="color:#' . $color . ';font-size:28px; display:block;">' . $fraudlabspro_status_display . '</span>
					</td>
				</tr>
				<tr>
					<td>
						<b>Transaction ID</b> <a href="javascript:;" class="flp-helper" title="Unique identifier for a transaction screened by FraudLabs Pro system."><span class="dashicons dashicons-editor-help"></span></a>
						<p><a href="https://www.fraudlabspro.com/merchant/transaction-details/' . $row->fraudlabspro_id . '" target="_blank">' . $row->fraudlabspro_id . '</a></p>
					</td>
				</tr>
				<tr>
					<td>
						<b>IP Address</b>
						<p>' . $row->ip_address . '</p>
					</td>
					<td>
						<b>IP Location</b> <a href="javascript:;" class="flp-helper" title="Location of the IP address."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . implode( ', ', $location ) . ' <a href="https://www.geolocation.com/' . $row->ip_address . '" target="_blank">[Map]</a></p>
					</td>
				</tr>
				<tr>
					<td>
						<b>IP Net Speed</b> <a href="javascript:;" class="flp-helper" title="Connection speed."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . $row->ip_netspeed . '</p>
					</td>
					<td>
						<b>IP ISP Name</b> <a href="javascript:;" class="flp-helper" title="ISP of the IP address."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . $row->ip_isp_name . '</p>
					</td>
				</tr>
				<tr>
					<td>
						<b>IP Domain</b> <a href="javascript:;" class="flp-helper" title="Domain name of the IP address."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . $row->ip_domain . '</p>
					</td>
					<td>
						<b>IP Usage Type</b> <a href="javascript:;" class="flp-helper" title="Usage type of the IP address. E.g, ISP, Commercial, Residential."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . ( ($row->ip_usage_type == 'NA' ) ? 'Not available [<a href="https://www.fraudlabspro.com/plan" target="_blank">Upgrade</a>]' : $row->ip_usage_type ) . '</p>
					</td>
				</tr>
				<tr>
					<td>
						<b>IP Time Zone</b> <a href="javascript:;" class="flp-helper" title="Time zone of the IP address."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . $row->ip_timezone . '</p>
					</td>
					<td>
						<b>IP Distance</b> <a href="javascript:;" class="flp-helper" title="Distance from IP address to Billing Location."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . ( ( $row->distance_in_km ) ? ( $row->distance_in_km . ' KM / ' . $row->distance_in_mile . ' Miles' ) : '-' ) . '</p>
					</td>
				</tr>
				<tr>
					<td>
						<b>IP Latitude</b> <a href="javascript:;" class="flp-helper" title="Latitude of the IP address."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . $row->ip_latitude . '</p>
					</td>
					<td>
						<b>IP Longitude</b> <a href="javascript:;" class="flp-helper" title="Longitude of the IP address."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . $row->ip_longitude . '</p>
					</td>
				</tr>
				<tr>
					<td>
						<b>High Risk Country</b> <a href="javascript:;" class="flp-helper" title="Whether IP address country is in the latest high risk country list."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . parse_fraud_result( $row->is_high_risk_country ) . '</p>
					</td>
					<td>
						<b>Free Email</b> <a href="javascript:;" class="flp-helper" title="Whether e-mail is from free e-mail provider."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . parse_fraud_result( $row->is_free_email ) . '</p>
					</td>
				</tr>
				<tr>
					<td>
						<b>Ship Forward</b> <a href="javascript:;" class="flp-helper" title="Whether shipping address is a freight forwarder address."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . parse_fraud_result( $row->is_address_ship_forward ) . '</p>
					</td>
					<td>
						<b>Using Proxy</b> <a href="javascript:;" class="flp-helper" title="Whether IP address is from Anonymous Proxy Server."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . parse_fraud_result( $row->is_proxy_ip_address ) . '</p>
					</td>
				</tr>
				<tr>
					<td>
						<b>BIN Found</b> <a href="javascript:;" class="flp-helper" title="Whether the BIN information matches our BIN list."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . parse_fraud_result( $row->is_bin_found ) . '</p>
					</td>
					<td>
						<b>Email Blacklist</b> <a href="javascript:;" class="flp-helper" title="Whether the email address is in our blacklist database."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . parse_fraud_result( $row->is_email_blacklist ) . '</p>
					</td>
				</tr>
				<tr>
					<td>
						<b>Credit Card Blacklist</b> <a href="javascript:;" class="flp-helper" title="Whether the credit card is in our blacklist database."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . parse_fraud_result( $row->is_credit_card_blacklist ) . '</p>
					</td>
					<td>
						<b>Balance</b> <a href="javascript:;" class="flp-helper" title="Balance of the credits available after this transaction."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . $row->fraudlabspro_credits . ' [<a href="https://www.fraudlabspro.com/plan" target="_blank">Upgrade</a>]</p>
					</td>
				</tr>
				<tr>
					<td colspan="3">
						<b>Message</b> <a href="javascript:;" class="flp-helper" title="FraudLabs Pro error message description."><span class="dashicons dashicons-editor-help"></span></a>
						<p>' . ( ( $row->fraudlabspro_message ) ? $row->fraudlabspro_error_code . ':' . $row->fraudlabspro_message : '-' ) . '</p>
					</td>
				</tr>
				<tr>
					<td colspan="3">
						<p>Please login to <a href="https://www.fraudlabspro.com/merchant/login" target="_blank">FraudLabs Pro Merchant Area</a> for more information about this order.</p>
					</td>
				</tr>
			</table>
			<form id="review-action" method="post">
				<p align="center">
					<input type="hidden" name="transactionId" value="' . $row->fraudlabspro_id . '" >
					<input type="hidden" name="orderId" value="' . $row->order_id . '" >
					<input type="hidden" id="new_status" name="new_status" value="" />
					<input type="hidden" id="feedback_note" name="feedback_note" value="" />';

			if( $row->fraudlabspro_status == 'REVIEW' ) {
				$table .= '
				<input type="submit" name="approve" id="approve-order" value="' . __( 'Approve', 'edd_fraudlabspro' ) . '" style="padding:10px 5px; background:#45b6af; border:1px solid #ccc; min-width:100px; cursor: pointer;" />
				<input type="submit" name="reject" id="reject-order" value="' . __( 'Reject', 'edd_fraudlabspro' ) . '" style="padding:10px 5px; background:#f3565d; border:1px solid #ccc; min-width:100px; cursor: pointer;" />';
			}

			if( !isset( $row->is_blacklisted ) ) {
				$table .= '
				<input type="button" name="reject-blacklist" id="reject-blacklist-order" value="' . __( 'Blacklist', 'edd_fraudlabspro' ) . '" style="padding:10px 5px; background:#000000; border:1px solid #ccc; min-width:100px; cursor: pointer;color:#fff" />';
			}

			$table .= '
				</p>
			</form>';

			echo '<div class="edd-fraudlabspro-payment-details"><h4>FraudLabs Pro</h4><blockquote>' . preg_replace( '/[\n]*/is', '', str_replace( '\'', '\\\'', $table ) ) . '</blockquote></div>';
			echo '
			<script>
			jQuery(function(){
				jQuery("#reject-blacklist-order").click(function(){
					var note = prompt("Please enter the reason(s) for blacklisting this order. (Optional)");
					if(note !== null){
						jQuery("#feedback_note").val(note);
						jQuery("#new_status").val("reject_blacklist");
						jQuery("#review-action").submit();
					}
				});
			});
			</script>';
		}
		else {
			echo '<div class="edd-fraudlabspro-payment-details"><h4>FraudLabs Pro</h4><blockquote>This transaction has not been screened by FraudLabs Pro.</blockquote></div>';
		}
	}
	else {
		echo '<div class="edd-fraudlabspro-payment-details"><h4>FraudLabs Pro</h4><blockquote>This transaction has not been screened by FraudLabs Pro.</blockquote></div>';
	}
}

// Get country name by country code.
function get_country_by_code( $code ) {
	$countries = array( 'AF' => 'Afghanistan','AL' => 'Albania','DZ' => 'Algeria','AS' => 'American Samoa','AD' => 'Andorra','AO' => 'Angola','AI' => 'Anguilla','AQ' => 'Antarctica','AG' => 'Antigua and Barbuda','AR' => 'Argentina','AM' => 'Armenia','AW' => 'Aruba','AU' => 'Australia','AT' => 'Austria','AZ' => 'Azerbaijan','BS' => 'Bahamas','BH' => 'Bahrain','BD' => 'Bangladesh','BB' => 'Barbados','BY' => 'Belarus','BE' => 'Belgium','BZ' => 'Belize','BJ' => 'Benin','BM' => 'Bermuda','BT' => 'Bhutan','BO' => 'Bolivia','BA' => 'Bosnia and Herzegovina','BW' => 'Botswana','BV' => 'Bouvet Island','BR' => 'Brazil','IO' => 'British Indian Ocean Territory','BN' => 'Brunei Darussalam','BG' => 'Bulgaria','BF' => 'Burkina Faso','BI' => 'Burundi','KH' => 'Cambodia','CM' => 'Cameroon','CA' => 'Canada','CV' => 'Cape Verde','KY' => 'Cayman Islands','CF' => 'Central African Republic','TD' => 'Chad','CL' => 'Chile','CN' => 'China','CX' => 'Christmas Island','CC' => 'Cocos (Keeling) Islands','CO' => 'Colombia','KM' => 'Comoros','CG' => 'Congo','CK' => 'Cook Islands','CR' => 'Costa Rica','CI' => 'Cote D\'Ivoire','HR' => 'Croatia','CU' => 'Cuba','CY' => 'Cyprus','CZ' => 'Czech Republic','CD' => 'Democratic Republic of Congo','DK' => 'Denmark','DJ' => 'Djibouti','DM' => 'Dominica','DO' => 'Dominican Republic','TP' => 'East Timor','EC' => 'Ecuador','EG' => 'Egypt','SV' => 'El Salvador','GQ' => 'Equatorial Guinea','ER' => 'Eritrea','EE' => 'Estonia','ET' => 'Ethiopia','FK' => 'Falkland Islands (Malvinas)','FO' => 'Faroe Islands','FJ' => 'Fiji','FI' => 'Finland','FR' => 'France','FX' => 'France, Metropolitan','GF' => 'French Guiana','PF' => 'French Polynesia','TF' => 'French Southern Territories','GA' => 'Gabon','GM' => 'Gambia','GE' => 'Georgia','DE' => 'Germany','GH' => 'Ghana','GI' => 'Gibraltar','GR' => 'Greece','GL' => 'Greenland','GD' => 'Grenada','GP' => 'Guadeloupe','GU' => 'Guam','GT' => 'Guatemala','GN' => 'Guinea','GW' => 'Guinea-bissau','GY' => 'Guyana','HT' => 'Haiti','HM' => 'Heard and Mc Donald Islands','HN' => 'Honduras','HK' => 'Hong Kong','HU' => 'Hungary','IS' => 'Iceland','IN' => 'India','ID' => 'Indonesia','IR' => 'Iran (Islamic Republic of)','IQ' => 'Iraq','IE' => 'Ireland','IL' => 'Israel','IT' => 'Italy','JM' => 'Jamaica','JP' => 'Japan','JO' => 'Jordan','KZ' => 'Kazakhstan','KE' => 'Kenya','KI' => 'Kiribati','KR' => 'Korea, Republic of','KW' => 'Kuwait','KG' => 'Kyrgyzstan','LA' => 'Lao People\'s Democratic Republic','LV' => 'Latvia','LB' => 'Lebanon','LS' => 'Lesotho','LR' => 'Liberia','LY' => 'Libyan Arab Jamahiriya','LI' => 'Liechtenstein','LT' => 'Lithuania','LU' => 'Luxembourg','MO' => 'Macau','MK' => 'Macedonia','MG' => 'Madagascar','MW' => 'Malawi','MY' => 'Malaysia','MV' => 'Maldives','ML' => 'Mali','MT' => 'Malta','MH' => 'Marshall Islands','MQ' => 'Martinique','MR' => 'Mauritania','MU' => 'Mauritius','YT' => 'Mayotte','MX' => 'Mexico','FM' => 'Micronesia, Federated States of','MD' => 'Moldova, Republic of','MC' => 'Monaco','MN' => 'Mongolia','MS' => 'Montserrat','MA' => 'Morocco','MZ' => 'Mozambique','MM' => 'Myanmar','NA' => 'Namibia','NR' => 'Nauru','NP' => 'Nepal','NL' => 'Netherlands','AN' => 'Netherlands Antilles','NC' => 'New Caledonia','NZ' => 'New Zealand','NI' => 'Nicaragua','NE' => 'Niger','NG' => 'Nigeria','NU' => 'Niue','NF' => 'Norfolk Island','KP' => 'North Korea','MP' => 'Northern Mariana Islands','NO' => 'Norway','OM' => 'Oman','PK' => 'Pakistan','PW' => 'Palau','PA' => 'Panama','PG' => 'Papua New Guinea','PY' => 'Paraguay','PE' => 'Peru','PH' => 'Philippines','PN' => 'Pitcairn','PL' => 'Poland','PT' => 'Portugal','PR' => 'Puerto Rico','QA' => 'Qatar','RE' => 'Reunion','RO' => 'Romania','RU' => 'Russian Federation','RW' => 'Rwanda','KN' => 'Saint Kitts and Nevis','LC' => 'Saint Lucia','VC' => 'Saint Vincent and the Grenadines','WS' => 'Samoa','SM' => 'San Marino','ST' => 'Sao Tome and Principe','SA' => 'Saudi Arabia','SN' => 'Senegal','SC' => 'Seychelles','SL' => 'Sierra Leone','SG' => 'Singapore','SK' => 'Slovak Republic','SI' => 'Slovenia','SB' => 'Solomon Islands','SO' => 'Somalia','ZA' => 'South Africa','GS' => 'South Georgia And The South Sandwich Islands','ES' => 'Spain','LK' => 'Sri Lanka','SH' => 'St. Helena','PM' => 'St. Pierre and Miquelon','SD' => 'Sudan','SR' => 'Suriname','SJ' => 'Svalbard and Jan Mayen Islands','SZ' => 'Swaziland','SE' => 'Sweden','CH' => 'Switzerland','SY' => 'Syrian Arab Republic','TW' => 'Taiwan','TJ' => 'Tajikistan','TZ' => 'Tanzania, United Republic of','TH' => 'Thailand','TG' => 'Togo','TK' => 'Tokelau','TO' => 'Tonga','TT' => 'Trinidad and Tobago','TN' => 'Tunisia','TR' => 'Turkey','TM' => 'Turkmenistan','TC' => 'Turks and Caicos Islands','TV' => 'Tuvalu','UG' => 'Uganda','UA' => 'Ukraine','AE' => 'United Arab Emirates','GB' => 'United Kingdom','US' => 'United States','UM' => 'United States Minor Outlying Islands','UY' => 'Uruguay','UZ' => 'Uzbekistan','VU' => 'Vanuatu','VA' => 'Vatican City State (Holy See)','VE' => 'Venezuela','VN' => 'Viet Nam','VG' => 'Virgin Islands (British)','VI' => 'Virgin Islands (U.S.)','WF' => 'Wallis and Futuna Islands','EH' => 'Western Sahara','YE' => 'Yemen','YU' => 'Yugoslavia','ZM' => 'Zambia','ZW' => 'Zimbabwe' );
	return ( isset( $countries[$code] ) ) ? $countries[$code] : NULL;
}

// Parse FraudLabs Pro API result.
function parse_fraud_result( $result ) {
	if ( $result == 'Y' )
		return 'Yes';

	if ( $result == 'N' )
		return 'No';

	if ( $result == 'NA' )
		return '-';

	return $result;
}

// Javascript agent.
function edd_fraudlabspro_add_javascript_agent() { 
	echo '<script>!function(){function t(){var t=document.createElement("script");t.type="text/javascript",t.async=!0,t.src="https://cdn.fraudlabspro.com/s.js";var e=document.getElementsByTagName("script")[0];e.parentNode.insertBefore(t,e)}window.attachEvent?window.attachEvent("onload",t):window.addEventListener("load",t,!1)}();</script>';
}

// Set up FraudLabs Pro Settings.
function edd_fraudlabspro_add_settings( $settings ) {
	$statuses = array();
	foreach( edd_get_payment_statuses() as $key => $status ) {
		$statuses[ $key ] = $status;
	}

	$fraudlabspro_settings = array(
		array(
			'id' => 'fraudlabspro_settings',
			'name' => '<strong>' . __( 'FraudLabs Pro Settings', 'edd_fraudlabspro' ) . '</strong>',
			'desc' => __( 'Configure FraudLabs Pro Settings', 'edd_fraudlabspro' ),
			'type' => 'header'
		),
		array(
			'id' => 'fraudlabspro_api_key',
			'name' => __( 'API Key', 'edd_fraudlabspro' ),
			'desc' => __( 'You can sign up for a free API key at <a href="https://www.fraudlabspro.com/pricing?utm_source=module&utm_medium=banner&utm_term=edd&utm_campaign=module%20banner" target="_blank">FraudLabs Pro</a>.', 'edd_fraudlabspro' ),
			'type' => 'text',
			'size' => 'regular'
		),
		array(
			'id' => 'fraudlabspro_approve_status',
			'name' => __( 'Approve Status', 'edd_fraudlabspro' ),
			'desc' => __( 'Change order status when order has been approved by FraudLabs Pro, or FraudLabs Pro <strong>Approve</strong> button has been pressed in order details page.', 'edd_fraudlabspro' ),
			'type' => 'select',
			'options' => $statuses
		),
		array(
			'id' => 'fraudlabspro_review_status',
			'name' => __( 'Review Status', 'edd_fraudlabspro' ),
			'desc' => __( 'Change order status when order has been marked as REVIEW by FraudLabs Pro.', 'edd_fraudlabspro' ),
			'type' => 'select',
			'options' => $statuses
		),
		array(
			'id' => 'fraudlabspro_reject_status',
			'name' => __( 'Reject Status', 'edd_fraudlabspro' ),
			'desc' => __( 'Change order status when order has been rejected by FraudLabs Pro, or FraudLabs Pro Reject button has been pressed in order details page.', 'edd_fraudlabspro' ),
			'type' => 'select',
			'options' => $statuses
		),
		array(
			'id' => 'fraudlabspro_test_ip',
			'name' => __( 'Test IP', 'edd_fraudlabspro' ),
			'desc' => __( 'Simulate visitor IP address. Leave this field blank for production run.', 'edd_fraudlabspro' ),
			'type' => 'text',
			'size' => 'regular'
		),
		array(
			'id' => 'fraudlabspro_debug_log',
			'name' => __( 'Debug Log', 'edd_fraudlabspro' ),
			'desc' => __( 'Enable Debug Log for development purposes.', 'edd_fraudlabspro' ),
			'type' => 'checkbox'
		)
	);

	return array_merge( $settings, $fraudlabspro_settings );
}

// Write to debug log to record details of process.
function write_debug_log($message) {
	global $edd_options;
	if( $edd_options['fraudlabspro_debug_log'] != 1 ) {
		return;
	}

	if( is_array( $message ) || is_object( $message ) ) {
		file_put_contents( FRAUDLABS_PRO_ROOT . 'debug.log', gmdate('Y-m-d H:i:s') . "\t" . print_r( $message, true ) . "\n", FILE_APPEND );
	} else {
		file_put_contents( FRAUDLABS_PRO_ROOT . 'debug.log', gmdate('Y-m-d H:i:s') . "\t" . $message . "\n", FILE_APPEND );
	}
}